/* (C) 2021 YiRing, Inc. */
package com.yiring.app.config;

import cn.dev33.satoken.exception.NotLoginException;
import com.yiring.app.constant.Code;
import com.yiring.app.exception.CodeException;
import com.yiring.common.core.I18n;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.exception.FailStatusException;
import javax.validation.ConstraintViolationException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.catalina.connector.ClientAbortException;
import org.aspectj.bridge.AbortException;
import org.hibernate.validator.internal.engine.ConstraintViolationImpl;
import org.springframework.http.HttpStatus;
import org.springframework.http.converter.HttpMessageNotWritableException;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * 全局错误处理
 *
 * @author ifzm
 * 2017年11月30日 上午11:36:31
 */
@Slf4j
@ControllerAdvice
@ResponseBody
@RequiredArgsConstructor
public class GlobalExceptionHandler {

    final I18n i18n;

    /**
     * 参数校验异常
     *
     * @param e 异常信息
     * @return 统一的校验失败信息 {@link Status#EXPECTATION_FAILED
     */
    @ExceptionHandler(
        value = { BindException.class, MethodArgumentNotValidException.class, ConstraintViolationException.class }
    )
    public Result<String> bindErrorHandler(Exception e) {
        String details = null;

        if (e instanceof ConstraintViolationException) {
            details = ((ConstraintViolationException) e).getConstraintViolations().iterator().next().getMessage();
        } else {
            BindingResult result = null;
            if (e instanceof MethodArgumentNotValidException) {
                result = ((MethodArgumentNotValidException) e).getBindingResult();
            } else if (e instanceof BindException) {
                result = ((BindException) e).getBindingResult();
            }

            if (result != null) {
                ObjectError error = result.getAllErrors().iterator().next();
                if (error instanceof FieldError fieldError) {
                    // TODO: 可以优化成提取 @ApiModelProperty value 中文描述
                    // 构建明确的字段错误提示, 例如: id 不能为 null, 如果自己填写了 message 则不追加 field 字段前缀
                    ConstraintViolationImpl<?> violation = error.unwrap(ConstraintViolationImpl.class);
                    String template = violation.getMessageTemplate();
                    String prefix = "";
                    // 如果是模板字符串, 则在消息前添加字段提示
                    if (template.startsWith("{") && template.endsWith("}")) {
                        prefix = fieldError.getField() + " ";
                    }

                    details = prefix + i18n.get(fieldError);
                } else {
                    details = i18n.get(error);
                }
            }
        }

        return Result.no(Status.EXPECTATION_FAILED, details);
    }

    /**
     * 不支持的HttpMethod异常
     *
     * @param e 异常信息
     * @return 异常信息反馈 {@link Status#METHOD_NOT_ALLOWED
     */
    @ExceptionHandler(HttpRequestMethodNotSupportedException.class)
    public Result<String> httpRequestMethodNotSupportedErrorHandler(Exception e) {
        return Result.no(Status.METHOD_NOT_ALLOWED, e.getMessage());
    }

    /**
     * 未登录异常（鉴权失败）
     *
     * @return 异常信息反馈 {@link Status#UNAUTHORIZED
     */
    @ExceptionHandler(NotLoginException.class)
    public Result<String> notLoginErrorHandler() {
        return Result.no(Status.UNAUTHORIZED);
    }

    /**
     * 自定义业务异常
     */
    @ExceptionHandler(CodeException.class)
    public Result<String> customCodeExceptionHandler(CodeException e) {
        Code code = e.getCode();
        return Result.no(Status.BAD_REQUEST, code.value(), code.reason(), null);
    }

    /**
     * 失败状态异常
     */
    @ExceptionHandler(FailStatusException.class)
    public Result<String> failStatusExceptionHandler(FailStatusException e) {
        return Result.no(e.getStatus(), e.getMessage());
    }

    /**
     * 取消请求异常（忽略）
     */
    @ExceptionHandler(
        value = { ClientAbortException.class, AbortException.class, HttpMessageNotWritableException.class }
    )
    public void clientAbortExceptionHandler() {}

    /**
     * 其他异常
     *
     * @param e 异常信息
     * @return 统一的500异常信息 {@link Status#INTERNAL_SERVER_ERROR
     */
    @ResponseStatus(code = HttpStatus.INTERNAL_SERVER_ERROR)
    @ExceptionHandler(Exception.class)
    public Result<String> defaultErrorHandler(Exception e) {
        log.error(e.getMessage(), e);
        return Result.no(Status.INTERNAL_SERVER_ERROR, e);
    }
}
