/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.upload;

import cn.hutool.core.io.FileUtil;
import com.yiring.common.core.Minio;
import com.yiring.common.service.UploadProcessService;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.List;
import javax.imageio.ImageIO;
import lombok.Cleanup;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.PDFRenderer;
import org.bytedeco.javacv.FFmpegFrameGrabber;
import org.bytedeco.javacv.Frame;
import org.bytedeco.javacv.Java2DFrameConverter;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;

/**
 * @author Jim
 * @version 0.1
 * 2022/9/23 16:44
 */

@ConditionalOnClass({ PDDocument.class, FFmpegFrameGrabber.class })
@Component
@RequiredArgsConstructor
public class UploadProcessServiceImpl implements UploadProcessService {

    final Minio minio;

    @Override
    public String handle(String object, InputStream is) {
        String suffix = FileUtil.getSuffix(object);

        // Image: 在文件名上追加图片物理像素
        if (isSupportiveImage(suffix)) {
            object = handleImage(object, is);
        }

        // PDF: 在文件名上追加页数，同时在同目录生成 PDF 每一页的图片
        if (isPdf(suffix)) {
            object = handlePdf(object, is);
        }

        // Video/Audio: 在文件名上追加时长，视频生成封面图
        if (isSupportiveMedia(suffix)) {
            object = handleMedia(object, suffix, is);
        }

        return object;
    }

    @SneakyThrows
    public String handleImage(String object, InputStream is) {
        BufferedImage image = ImageIO.read(is);
        return fillSuffix(object, image.getWidth() + "x" + image.getHeight());
    }

    @SneakyThrows
    public String handlePdf(String object, InputStream is) {
        PDDocument doc = PDDocument.load(is);
        int pages = doc.getNumberOfPages();

        // 构建具有 PDF 页数标记的存储地址
        String filepath = fillSuffix(object, "P" + pages);

        // 将 PDF 解析成图片上传
        String format = "jpg";
        PDFRenderer renderer = new PDFRenderer(doc);
        for (int i = 0; i < pages; i++) {
            @Cleanup
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            BufferedImage image = renderer.renderImageWithDPI(i, 144);
            ImageIO.write(image, format, os);
            @Cleanup
            InputStream io = new ByteArrayInputStream(os.toByteArray());
            minio.putObject(io, MediaType.IMAGE_JPEG_VALUE, filepath + "." + (i + 1) + "." + format);
        }

        return filepath;
    }

    @SneakyThrows
    public String handleMedia(String object, String suffix, InputStream is) {
        // 判断是否为视频
        @Cleanup
        FFmpegFrameGrabber ff = new FFmpegFrameGrabber(is);
        ff.start();

        // 构建具有时长（秒）标记的存储地址
        String filepath = fillSuffix(object, "T" + (ff.getLengthInTime() / (1000 * 1000)));

        // 视频截取首帧可见画面作为封面
        if (isSupportiveVideo(suffix)) {
            // 获取视频可见画面帧
            Frame frame = getPictureFrame(ff);

            // 将截取的封面并存储
            String format = "jpg";
            @Cleanup
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            ImageIO.write(frameToBufferedImage(frame), format, os);
            @Cleanup
            InputStream io = new ByteArrayInputStream(os.toByteArray());
            minio.putObject(io, MediaType.IMAGE_JPEG_VALUE, filepath + "." + format);
        }

        ff.stop();
        return filepath;
    }

    public static Frame getPictureFrame(FFmpegFrameGrabber grabber) throws FFmpegFrameGrabber.Exception {
        int ftp = grabber.getLengthInFrames();
        int flag = 0;
        Frame frame = null;
        while (flag <= ftp) {
            // 获取帧
            frame = grabber.grabImage();
            // 过滤前3帧，避免出现全黑图片
            if ((flag > 3) && (frame != null)) {
                break;
            }
            flag++;
        }
        return frame;
    }

    public static RenderedImage frameToBufferedImage(Frame frame) {
        @Cleanup
        Java2DFrameConverter converter = new Java2DFrameConverter();
        return converter.getBufferedImage(frame);
    }

    public static boolean isSupportiveMedia(String suffix) {
        return isSupportiveVideo(suffix) || isSupportiveAudio(suffix);
    }

    public static boolean isSupportiveVideo(String suffix) {
        return List.of("mp4", "flv", "avi", "rmvb", "rm", "wmv", "mkv", "mpg", "mpeg").contains(suffix.toLowerCase());
    }

    public static boolean isSupportiveAudio(String suffix) {
        return List.of("mp3", "wav").contains(suffix.toLowerCase());
    }

    public static boolean isSupportiveImage(String suffix) {
        return List.of("png", "jpg", "webp", "gif", "tif", "svg", "bmp").contains(suffix.toLowerCase());
    }

    public static boolean isPdf(String suffix) {
        return "pdf".equalsIgnoreCase(suffix);
    }

    public static String fillSuffix(String object, String fill) {
        String suffix = FileUtil.getSuffix(object);
        String regex = "^(.*)\\." + suffix + "$";
        return object.replaceAll(regex, "$1." + fill + "." + suffix);
    }
}
