/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.web.auth;

import cn.dev33.satoken.secure.SaSecureUtil;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.yiring.auth.domain.user.User;
import com.yiring.auth.domain.user.UserRepository;
import com.yiring.auth.param.auth.LoginParam;
import com.yiring.auth.param.auth.RegisterParam;
import com.yiring.auth.vo.auth.LoginVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.util.Commons;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.time.LocalDateTime;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Example;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 用户认证控制器
 *
 * @author Jim
 * @version 0.1
 * 2022/1/24 14:13
 */

@Slf4j
@Validated
@SuppressWarnings({ "deprecation", "all" })
@ApiSupport(order = -99)
@Api(tags = "身份认证", description = "Auth")
@RestController
@RequestMapping("/auth/")
public class AuthController {

    @Resource
    UserRepository userRepository;

    @ApiOperation(value = "注册")
    @PostMapping("register")
    public Result<String> register(@Valid RegisterParam param) {
        // 检查用户名是否存在
        long count = userRepository.count(Example.of(User.builder().username(param.getUsername()).build()));
        if (count > 0) {
            throw Status.BAD_REQUEST.exception("Code.100000");
        }

        // 检查手机号是否存在
        count = userRepository.count(Example.of(User.builder().mobile(param.getMobile()).build()));
        if (count > 0) {
            throw Status.BAD_REQUEST.exception("Code.100001");
        }

        // 检查邮箱是否存在
        if (StrUtil.isNotBlank(param.getEmail())) {
            count = userRepository.count(Example.of(User.builder().email(param.getEmail()).build()));
            if (count > 0) {
                throw Status.BAD_REQUEST.exception("Code.100002");
            }
        }

        // 构建用户信息写入数据库
        User user = User
            .builder()
            .avatar(param.getAvatar())
            .mobile(param.getMobile())
            .realName(param.getRealName())
            .username(param.getUsername())
            .password(SaSecureUtil.sha256(param.getPassword()))
            .enabled(param.getEnable())
            .createTime(LocalDateTime.now())
            .build();
        userRepository.saveAndFlush(user);
        return Result.ok();
    }

    @ApiOperation(value = "登录")
    @PostMapping("login")
    public Result<LoginVo> login(@Valid LoginParam param, HttpServletRequest request) {
        // 查询用户信息是否匹配
        User user = userRepository.findByAccount(param.getAccount());
        if (user == null) {
            throw Status.BAD_REQUEST.exception("Code.100003");
        }

        // 检查密码
        String cps = SaSecureUtil.sha256(param.getPassword());
        if (!cps.equals(user.getPassword())) {
            throw Status.BAD_REQUEST.exception("Code.100003");
        }

        // 检查用户是否已被删除
        if (user.getDeleteTime() != null) {
            throw Status.BAD_REQUEST.exception("Code.100004");
        }

        // 检查用户是否被允许登录
        if (!Boolean.TRUE.equals(user.getEnabled())) {
            throw Status.BAD_REQUEST.exception("Code.100005");
        }

        // 更新用户信息
        user.setLastLoginIp(Commons.getClientIpAddress(request));
        user.setLastLoginTime(LocalDateTime.now());
        userRepository.saveAndFlush(user);

        // 登录
        StpUtil.login(user.getId());

        // 构建用户所需信息
        LoginVo vo = LoginVo.builder().userId(user.getId()).token(StpUtil.getTokenValue()).build();
        return Result.ok(vo);
    }

    @ApiOperation(value = "检查登录状态")
    @GetMapping("valid")
    public Result<Boolean> valid() {
        return Result.ok(StpUtil.isLogin());
    }

    @ApiOperation(value = "登出")
    @GetMapping("logout")
    public Result<String> logout() {
        StpUtil.logout();
        return Result.ok();
    }
}
